## Linux 技能篇
### 1、工作中常用的 Linux 命令有哪些？
```shell
- 文件目录操作：cd、ls、pwd、touch、mkdir、mv 、rm
- 搜索命令：whereis、find、which
- 文本操作：cat/tac、head/tail、wc、cut、sort、uniq、grep、awk、sed
- 系统操作：ps、top、ifconfig、ping

cat # 查看文件，一次性输出
tac # 一次性反着输出文件内容
more # 单页形式查看文件
less # 众所周知，少即是多less is more，所以和more一样
vim # 是一个文件编辑工具，可以编辑文件
head # 从头读取部分文件内容
tail # 从末尾读取部分文件内容
echo # 输出特定内容，可以以文件作为输入即可查看文件内容
nl # 以带行号的形式输出文件内容
grep # 强大的文本搜索工具，以文件作为输入且将搜索条件设置为""即可查看文件内容
```
### 2、Linux命令，输出当前目录下文件的数量
```shell
# 使用`wc`命令统计`ls`命令输出的行数，也就是当前目录下的文件数量
ls | wc -l
```
### 3、Linux命令，查询并输出Linux系统中带有“gift”的所有行
```shell
# -i选项忽略大小写
grep "gift" filename
grep -i "gift" filename
# 需要输出包含“gift”行所在的文件及行号，可以使用`-n`选项
grep -in "gift" filename
```
### 4、Linux下如何检索是否有某个文件
```python
# 在Linux下，可以使用 `find` 命令来检索是否有某个文件存在
find [path] -name [filename]
# 例如，要在当前目录下查找是否存在文件名为 `example.txt` 的文件，可以使用以下命令
find . -name example.txt
```
### 5、找出一个文件前10行
```bash
head -n 10 file_name

说明：
- `head` 指令用于从文件头部向后读取内容，未指定行数时默认显示前10行
- 可选参数 `-n lines`，可指定只显示文件的前 `lines` 行内容
- `file_name` 为被读取的日志文件
```
### 6、找出一个文件第10行，并打印出来
可以使用 `sed` 命令来寻找文件的第10行并打印出来。具体的命令如下：
```bash
sed -n '10p' filename
```
其中，`-n` 表示只输出指定的行，`'10p'` 表示输出第10行，`filename` 表示要查找的文件名。
如果想查找的文件很大，可以使用 `head` 命令进行优化，如下所示：
```bash
head -n 10 filename | tail -n 1
```
其中，`head -n 10` 表示输出前10行，`tail -n 1` 表示输出最后一行，因此就输出了第10行。
### 7、找出一个文件最后（最新的）10行，并打印出来
可以使用 `tail` 命令来寻找文件的最后10行并打印出来。具体的命令如下：
```bash
tail -n 10 filename
```
其中，`-n 10` 表示输出文件的最后10行，`filename` 表示要查找的文件名。
如果想要在输出的结果中同时显示行号，可以使用 `-n` 选项，如下所示：
```bash
tail -n 10 filename | cat -n
```
其中，`cat -n` 表示输出行号，这样就可以在结果中显示行号了。
### 8、找一个文件第5行到第15行之间的内容，并打印出来
除了之前提到的 `cat` 和 `sed` 命令、`head` 和 `tail` 命令，还可以使用其他命令来实现查看文件中指定行范围的内容，以下是一些例子：
1. 使用 `awk` 命令
```bash
   awk 'NR>=5 && NR<=15' filename
```
   上述命令中，`filename` 表示需要查看的文件名，`5` 表示需要查看的起始行号，`15` 表示需要查看的结束行号。该命令使用 `awk` 命令来查找文件中第 5 行到第 15 行之间的内容并输出到标准输出。其中，`NR` 表示当前处理的行号，`>=` 和 `<=` 表示大于等于和小于等于。
2. 使用 `sed` 命令
```bash
   sed -n '5,15p' filename
```
   上述命令中，`filename` 表示需要查看的文件名，`5` 表示需要查看的起始行号，`15` 表示需要查看的结束行号。该命令使用 `sed` 命令来查找文件中第 5 行到第 15 行之间的内容并输出到标准输出。其中，`-n` 参数表示关闭默认输出行为，`'5,15p'` 参数指定需要输出文件的第 5 行到第 15 行的内容。
3. 使用 `tail` 和 `head` 命令
```bash
   tail -n +5 filename | head -n 11
```
   上述命令中，`filename` 表示需要查看的文件名，`5` 表示需要查看的起始行号，`11` 表示需要查看的行数，即第 5 行到第 15 行之间的内容。该命令首先使用 `tail` 命令输出文件从第 5 行开始到最后一行的内容，并使用 `head` 命令输出前 11 行内容，即第 5 行到第 15 行之间的内容。
### 9、Linux 中如何查看内存使用率，并说明命令与参数的具体含义
在 Linux 中，可以使用`free`命令来查看内存使用率。该命令的默认用法显示了系统当前的内存使用情况。  
命令： `free [options]`
其中，`options` 是可选参数，可以根据需要选择使用。以下是一些常用的参数：  
`-h` ：以人类可读的格式来显示内存大小（以单位为 K、M、G 等表示）。例如： `free -h` 。  
`-b` ：以字节为单位显示内存大小。例如： `free -b` 。  
`-m` ：以 Mebibyte（MiB）为单位显示内存大小。例如： `free -m` 。  
`-g` ：以 Gibibyte（GiB）为单位显示内存大小。例如： `free -g` 。
- `top` 
	`top` 命令监视进程和Linux整体性能，可以看到内存使用情况
- `free -k -c 5` 
	`free`是专门查看当前系统内存的使用情况的工具，-k 是以千字节为单位，-c N 是按设定或默认的时间间隔重复打印，直到 N 次后自动退出
- `vmstat -s`
	`vmstat`(虚拟内存统计)指令能对虚拟内存、进程、CPU等的整体情况进行监视，-s 显示详情
- `cat /proc/meminfo` 
	这个文件保存着内核使用内存情况的各种信息。是了解Linux系统内存使用状况的主要接口，`free`、`vmstat`等命令就是通过它获取数据的

### 10、Linux中，对linux的IO消耗、内存、磁盘空间如何查询
在Linux中，有很多命令可以用来检查系统的IO消耗、内存使用情况以及磁盘空间。下面是一些常用的命令：
1. **IO消耗**：可以使用`iostat`命令。这个命令可以展示系统级别的I/O操作信息。一个例子是：
```bash
iostat -xd
```
这个命令会显示所有的磁盘设备的I/O统计信息。
2. **内存使用情况**：可以使用`free`命令。这个命令可以展示系统的内存使用情况。一个例子是：
```bash
free -h
```
这个命令会以易读的方式（如KB、MB、GB）显示系统的内存使用情况。
3. **磁盘空间**：可以使用`df`命令。这个命令可以展示文件系统的磁盘空间使用情况。一个例子是：
```bash
df -h
```
这个命令会以易读的方式（如KB、MB、GB）显示文件系统的磁盘空间使用情况。
请注意，这些命令可能需要以超级用户身份（即使用`sudo`）来运行，这取决于你的系统设置。
### 11、Linux，如何查看日志里面有 error 关键字的日志记录，与这个记录前后三行的日志信息？请给出命令，并对命令进行说明
```shell
grep -C 3 "keyword" file_path

grep：文本搜索工具，可以接收管道传来的数据，这里是读取的文件的数据  
-C n：将匹配到关键字的前后n行一同输出  
"keyword"：预期被匹配到的关键字，在egrep或-E参数作用下，可以使用正则表达式  
file_path：将文件内容作为输入
```
### 12、Linux如何实时查看日志记录
步骤：
- `tail -f`日志：默认最后10行，相当于增加参数 -n 10
- `tail -200f` 日志：最后200行，某一时刻往前推
注意事项：
- 有时候，对于一些大型的日志文件， `tail -f` 命令可能会很慢。  
- 在这种情况下，可以考虑使用`less`命令，将其配合管道操作符`|`使用。  
- 类似这样：`less +F <file>`。这将会在 less 中打开文件，并切换到跟随模式，实现实时查看日志的效果。  
在`less`模式下，按下`Ctrl + C`可以停止实时查看，并退回到`less`的浏览模式
### 13、Linux 怎么杀死进程？并对相应命令给出说明
步骤：
1. 通过 `ps` 指令获取进程ID
2. 通过 `kill -9 $PID` 命令杀死进程号为$PID的进程
`kill` 命令用于删除执行中的程序或工作，`kill` 杀死程序的的方式很多，可输入 `kill -l` 查看死法列表，其中 `-9` 代表以第9个信号去杀死程序，等同于 `kill -kill`
注意：需要注意的是，**使用 kill 命令可能会导致未保存的数据丢失**，因此请谨慎使用
### 14、在Linux命令中，ps -ef 与 ps -au 有什么区别，请详细说明
`ps`命令是Process Status的缩写，==**用来列出系统中当前运行的进程**==。
- 使用该命令可以确定有哪些进程正在运行和运行的状态、进程是否结束、进程有没有僵死、哪些进程占用了过多的资源等等。  
- ps 命令所列出的进行是当前进程的快照，也就是并不是动态的，而是执行该命令时那一时刻进行的状态。

相关参数说明
- `-A`: 显示所有进程。
- `a`: 显示一个终端的所有进程，除了会话引线
- `u`: uid or username 选择有效的用户id或者是用户名
- `-e`:此参数的效果和指定"A"参数相同。
- `-f`: 全部列出，通常和其他选项联用。

1. ps -ef用于查看全格式的全部进程，ps -aux也是用于查看进程。
2. ps -ef和ps aux，这两者的输出结果差别不大，但是展示样式不一样
    - aux是BSD风格，显示的项目有：USER , PID , %CPU , %MEM , VSZ , RSS , TTY , STAT , START , TIME , COMMAND
    - -ef是System V风格，显示的项目有：UID , PID , PPID , C , STIME , TTY , TIME , CMD。
3. COMMADN列如果过长，aux会截断显示，而ef不会。
### 15、简述一下Linux 的打包指令
在 Linux 中，有许多不同的指令可以用于打包文件和目录。下面是一些常见的 Linux 打包指令。  
1. tar 命令：用于打包文件和目录，常用于创建和提取 tar 归档文件。  
2. 打包命令：`tar -cvf filename`  
以下是一些常用的选项：  
- `-c` ：创建 tar 归档文件。
- `-x` ：解压 tar 归档文件。
- `-f <file>` ：指定要操作的归档文件。
- `-z` ：使用 gzip 进行压缩和解压缩。
- `-j` ：使用 bzip2 进行压缩和解压缩。
- `-v` ：显示操作的详细信息。
- `-C <directory>` ：改变目录，指定要解压到的目录。

例如，创建一个 tar 归档文件：`tar -czvf archive.tar.gz myfile.txt mydir`
2. zip 命令：用于创建和解压 zip 格式的归档文件。  
以下是一些常用的选项：  
- `-r` ：递归压缩目录。
- `-q` ：安静模式，不显示操作的详细信息。

例如，创建一个 zip 归档文件：`zip -r archive.zip myfile.txt mydir`

3. gzip 命令：用于压缩文件，并以.gz 扩展名保存压缩文件。只能处理单个文件，不能压缩目录。 
以下是一些常用的选项：  
- `-r` ：递归压缩目录及其内容。
- `-d` ：解压缩文件。

例如，压缩文件：`gzip myfile.txt`
- tar 和 gzip 结合使用的压缩和解压缩指令：
- 压缩文件或目录：`tar -czvf archive.tar.gz myfile.txt mydir`
- 解压缩文件或目录：`tar -xzvf archive.tar.gz`

注意事项  
这些指令提供了灵活的选项，能够满足不同的打包需求。具体使用哪个指令取决于你要打包的内容和需求。
### 16、Linux命令，如何使用Linux命令杀死后台执行的任务，并详细说明参数的作用。
使用方法  
在Linux中，可以使用`kill`命令杀死后台执行的任务。  
`kill`命令的语法如下：  
命令： `kill [options] <pid>`
其中，`pid`表示进程的 ID，可通过`ps`命令查看。  
常用的`kill`命令选项和其作用如下：  
- `-s <signal>` ：指定要发送的信号，默认为`TERM` （15），可通过`kill -l`命令查看可用的信号列表。  
- `-l` ：列出可用的信号名称。  
- `-a` ：发送信号到所有进程，但不包括自己和 init 进程。  
- `-p` ：指定要发送信号的进程 ID。  
- `-SIGNAL` ：与`-s <signal>`相同，但信号名称前面要加上`SIG`前缀。例如，`kill -SIGHUP`。  
- `-l <signal>`：查询信号名称相对应的信号值。例如，`kill -l HUP`。

注意事项  
需要注意的是，使用 kill 命令可能会导致未保存的数据丢失，因此请谨慎使用。  
### 17、Linux中硬链接和软链接的区别
1、定义不同
软链接又叫符号链接，这个文件包含了另一个文件的路径名。可以是任意文件或目录，可以链接不同文件系统的文件。  
硬链接就是一个文件的一个或多个文件名。把文件名和计算机文件系统使用的节点号链接起来。因此我们可以用多个文件名与同一个文件进行链接，这些文件名可以在同一目录或不同目录。
2、限制不同
硬链接只能对已存在的文件进行创建，不能对目录进行创建，也不能交叉文件系统进行硬链接的创建；  
软链接可对文件或目录创建，支持交叉文件系统，还可对不存在的文件或目录创建软链接；
3、影响不同
删除一个硬链接文件并不影响其他有相同inode号的文件。而删除软链接并不影响被指向的文件，但若被指向的原文件被删除，则相关软连接被称为死链接。
4、命令不同
创建软链接使用`ln -s 源文件 目标文件`，注意其中源文件必须是绝对路径；创建软链接使用`ln 源文件 目标文件`。
硬链接：ln source target  
软链接：ln -s source target
### 18、Linux有一个日志，需要通过Linux命令输出第二列的内容，如何实现？
方法  
要通过 Linux 命令输出日志文件中的第二列内容，可以使用`awk`命令来实现。  
`awk`是一种文本处理工具，可以按照指定规则从文本文件中抽取数据并对其进行处理。
命令： `awk '{print $2}'` 日志文件名  
- 在上面的命令中，你需要将`日志文件名`替换为你实际的日志文件名。  
- 然后，`$2`表示提取每行的第二列。  
-  `print` 命令将提取到的第二列内容打印出来。
### 19、Linux中，找一个文件里面出现超过10个的单词，单词一行一个
```shell
sort yourfile.txt | uniq -c | awk '$1 > 10 {print $2}'
```
这条命令的组成部分解释如下：
1. `sort yourfile.txt`：首先将文件`yourfile.txt`中的内容进行排序。这是必要的，因为`uniq`命令只能对连续出现的行进行计数。
2. `uniq -c`：`uniq`命令用于过滤重复的行，`-c`选项表示在每个单词前显示该行重复出现的次数。
3. `awk '$1 > 10 {print $2}'`：使用`awk`命令过滤出那些出现次数超过10次的单词。`$1`代表每行的第一个字段（即单词出现的次数），`$2`代表第二个字段（即单词本身）。如果第一个字段的值大于10，`awk`会打印出第二个字段的值，即出现次数超过10次的单词。
### 20、Linux中， 两个文件不一样，怎么对比
在Linux中，如果你想要对比两个文件的不同，可以使用`diff`命令。`diff`命令会逐行比较两个文件的内容，并输出它们之间的差异。
以下是`diff`命令的基本用法：
```bash
diff file1 file2
```
这将会输出`file1`和`file2`之间的所有差异。差异会以特定的格式显示，例如：
- `<` 表示该行仅出现在第一个文件中。
- `>` 表示该行仅出现在第二个文件中。
- `c` 表示两个文件在该处有所不同，并会显示具体的差异。
如果你想要更直观地查看差异，可以使用`-u`（统一格式）选项，这样`diff`会以更易于理解的方式显示差异：
```bash
diff -u file1 file2
```
另外，如果你想要忽略大小写差异，可以使用`-i`选项：
```bash
diff -i file1 file2
 ```
如果你只关心两个文件是否不同，而不关心具体的差异内容，可以使用`cmp`命令：
```bash
cmp file1 file2
```
如果`cmp`命令没有输出任何内容，那么这两个文件是相同的。否则，它会显示第一个发现差异的位置。
### 21、Linux命令，查找大于 1M 的文件
```shell
find . -type f -size +1M
```
这里的参数解释如下：
- `.`：表示当前目录。你也可以替换为其他目录的路径，以在该目录下进行搜索。
- `-type f`：表示只查找文件（不包括目录）。
- `-size +1M`：表示查找大于1M的文件。`+1M`中的`+`表示“大于”，`1M`表示1兆字节（Megabyte）。
### 22、Linux 查找当前目录下所有后缀为.py 的文件
在Linux中，你可以使用`find`命令来查找当前目录下所有后缀为`.py`的文件。以下是相应的命令：
```bash
find . -type f -name "*.py"
```
这里的参数解释如下：
- `.`：表示当前目录。
- `-type f`：表示只查找文件（不包括目录）。
- `-name "*.py"`：表示查找文件名以`.py`结尾的文件。引号是为了防止shell在命令执行之前对星号进行扩展。
执行这个命令后，你会得到当前目录及其所有子目录下所有后缀为`.py`的文件的列表。如果你只想在当前目录查找，不想递归到子目录，可以使用`maxdepth`选项限制搜索深度：
```bash
find . -maxdepth 1 -type f -name "*.py"
```
`-maxdepth 1`表示只在当前目录层级搜索，不进入更深层的目录。
### 23、Linux中，系统操作卡顿怎么分析原因，会有哪些命令
在Linux系统中，如果遇到操作卡顿的情况，可以通过一系列命令和步骤来分析原因。以下是一些常用的方法和命令：
#### 一、查看系统资源使用情况
1. **CPU和内存使用情况**
    - **top命令**：实时显示系统中各个进程的资源占用情况，包括CPU和内存使用率。这是最常用的系统监控工具之一。
    - **htop命令**：htop是top命令的一个增强版本，提供了一个更友好的用户界面，可以更方便地管理进程。
2. **内存使用情况**
    - **free命令**：显示系统的总内存、已用内存、空闲内存等信息。使用`free -h`可以以易读的格式（如KB、MB、GB）显示内存使用情况。
3. **磁盘使用情况**
    - **df命令**：查看文件系统的磁盘空间占用情况。使用`df -h`可以方便地以易读格式显示信息。
    - **du命令**：查看目录或文件的磁盘使用情况。例如，`du -sh /path/to/directory`可以显示指定目录的总大小。
#### 二、监控磁盘I/O性能
- **iostat命令**：用于监控系统输入输出设备和CPU的使用情况。通过`iostat -x 1`可以每隔1秒刷新一次磁盘I/O统计信息，帮助识别磁盘性能瓶颈。
- **iotop命令**：实时显示哪些进程正在使用磁盘I/O，以及它们的使用量。这有助于识别哪些进程可能导致了磁盘I/O性能问题。
#### 三、检查网络连接
- **ping命令**：用于测试网络连接情况。通过向目标主机发送ICMP回显请求数据包来检测网络延迟和丢包率。
- **netstat命令**（或更现代的ss命令）：查看网络连接、路由表、接口统计等信息。这有助于识别网络流量异常或连接问题。
#### 四、查看系统日志
- 系统日志通常位于`/var/log`目录下，如`/var/log/messages`、`/var/log/syslog`等。这些日志文件中可能包含与卡顿相关的错误信息、警告或异常事件。
- 使用`dmesg`命令查看内核环形缓冲区中的消息，这些消息可能包含硬件故障、驱动问题或模块加载失败等信息。
#### 五、其他命令和步骤
- **ps命令**：查看当前运行的进程及其状态。可以使用`ps aux`来显示所有进程的详细信息。
- **strace命令**：跟踪一个进程的系统调用和接收的信号。如果怀疑某个进程导致系统卡顿，可以使用`strace -p <PID>`来跟踪该进程。
- **ulimit命令**：查看或设置shell进程的资源限制，如最大文件数、打开文件描述符数等。使用`ulimit -a`可以查看当前会话的资源限制。
#### 六、综合分析和优化
- 根据以上命令和步骤收集到的信息，综合分析系统卡顿的可能原因。
- 针对不同的原因采取相应的优化措施，如升级硬件、优化软件配置、释放系统资源、修复文件系统或网络问题等。
请注意，解决Linux系统卡顿问题可能需要一定的系统管理和故障排查经验。如果问题复杂或难以自行解决，建议咨询Linux社区或寻求专业技术支持。
### 24、Linux中，监控操作系统的常用命令
在Linux中，监控操作系统的常用命令种类繁多，它们可以帮助系统管理员或用户实时了解系统的运行状态、资源使用情况以及潜在的性能瓶颈。以下是一些常用的监控命令及其功能简介：
#### 1. top
**功能**：top命令是Linux下最常用的性能监控工具之一，能够实时显示系统的总体运行情况以及各个进程的资源占用情况。通过top命令，用户可以查看CPU、内存的使用情况，以及进程的详细信息，如PID、用户、优先级、虚拟内存、物理内存等。
**使用方法**：在终端中输入`top`命令后回车即可。top命令提供了交互式界面，用户可以通过按键操作来排序、过滤进程信息或改变显示内容。
#### 2. vmstat
**功能**：vmstat命令用于显示关于系统虚拟内存、进程、CPU活动等的信息。它可以帮助用户了解系统的内存管理、进程调度以及IO等待情况。
**使用方法**：在终端中输入`vmstat`命令，并可选地加上时间间隔参数（如`vmstat 1`表示每秒刷新一次）来持续监控。
#### 3. sar
**功能**：sar命令是System Activity Reporter的缩写，用于收集、报告或保存系统活动信息。它可以提供CPU使用率、内存使用情况、磁盘IO、网络流量等多种系统性能数据的历史统计信息。
**使用方法**：sar命令的用法较为灵活，可以通过不同的选项来收集不同类型的性能数据。例如，`sar -u 1`可以每秒收集一次CPU使用情况的数据。
#### 4. iostat
**功能**：iostat命令主要用于监控系统输入输出设备和CPU的使用情况。通过iostat命令，用户可以获取磁盘的读写速度、IOPS（每秒输入输出次数）、平均响应时间等信息，从而了解磁盘的性能状况。
**使用方法**：在终端中输入`iostat`命令，并可选地加上时间间隔和磁盘设备参数来监控特定磁盘的性能。
#### 5. netstat
**功能**：netstat命令用于显示网络连接、路由表、接口统计等信息。通过netstat命令，用户可以查看系统的网络连接状态、监听端口情况等，从而诊断网络问题。
**使用方法**：在终端中输入`netstat`命令，并可选地加上不同的选项来过滤显示的信息。例如，`netstat -tuln`可以显示所有处于监听状态的TCP和UDP端口。
#### 6. iftop
**功能**：iftop命令是一个实时的网络带宽监控工具，用于显示网络接口上的流量情况。它可以按主机或端口显示带宽使用情况，帮助用户识别占用大量网络带宽的进程或应用。
**使用方法**：在终端中输入`iftop`命令，并可选地加上网络接口参数来监控特定接口的流量。
#### 7. htop
**功能**：htop是top命令的一个增强版本，提供了一个更加友好和交互式的用户界面。它除了显示top命令所能提供的信息外，还支持颜色高亮显示、垂直和水平滚动查看进程信息等功能。
**使用方法**：在终端中输入`htop`命令后回车即可。用户可以通过鼠标点击或键盘操作来管理进程、排序显示内容等。
#### 8. free
**功能**：free命令用于显示系统的内存使用情况，包括物理内存、交换空间以及缓存的使用情况。通过free命令，用户可以快速了解系统的内存是否充足，以及是否有必要进行内存优化。
**使用方法**：在终端中输入`free`命令，并可选地加上`-m`（以MB为单位显示）或`-g`（以GB为单位显示）等选项来改变显示单位。
#### 9. df 和 du
**功能**：
- df命令用于显示磁盘分区的使用情况，帮助用户了解磁盘空间是否足够。
- du命令用于估计文件或目录的磁盘空间使用量，帮助用户找出占用大量磁盘空间的文件或目录。
**使用方法**：
- `df -h`命令以易读的格式显示磁盘分区的使用情况。
- `du -sh /path/to/directory`命令显示指定目录的总大小（其中`-s`表示汇总大小，`-h`表示以易读的格式显示）。
这些命令共同构成了Linux系统监控的强大工具集，通过它们，系统管理员和用户可以实时了解系统的运行状态，及时发现并解决潜在的性能问题。
### 25、Linux中，查看 JVM 的命令
在Linux中，查看Java虚拟机（JVM）的状态和性能参数，通常会使用JDK自带的命令行工具，如`jps`、`jstat`、`jinfo`、`jmap`和`jstack`等。这些工具都位于JDK的`bin`目录下，因此，在使用前需要确保JDK已正确安装，并且环境变量已配置妥当。
以下是一些常用的JVM查看命令：
1. **jps（Java Virtual Machine Process Status Tool）**：  
    `jps`命令与Linux的`ps`命令类似，用于列出当前机器上所有的Java进程。使用此命令，你可以轻松找到Java应用程序的进程ID（PID），这是后续使用其他JVM工具进行进一步分析的基础。
```bash
jps -l
```
2. **jstat（Java Virtual Machine Statistics Monitoring Tool）**：  
    `jstat`命令用于监视JVM的各种运行时状态信息，包括类加载、内存使用、垃圾回收等。
```bash
jstat -gcutil <pid> 1000
```
    上述命令会每1000毫秒（即1秒）输出一次指定Java进程（PID）的垃圾回收统计信息。
3. **jinfo（Java Configuration Info）**：  
    `jinfo`命令用于打印出指定Java进程的配置信息，包括JVM参数和系统属性。
```bash
jinfo <pid>
```
4. **jmap（Java Memory Map）**：  
    `jmap`命令用于生成Java堆内存的详细映射信息，包括Java对象在堆内存中的布局。
```bash
jmap -histo <pid>
```
    上述命令会打印出指定Java进程的堆内存中的对象统计信息。
5. **jstack（Java Stack Trace）**：  
    `jstack`命令用于打印出指定Java进程的线程堆栈信息，这对于分析线程状态、死锁等问题非常有用。
```bash
jstack <pid>
```
使用这些命令时，请确保你有足够的权限来访问目标Java进程。在生产环境中，通常需要具有与目标Java进程相同的用户权限，或者使用`sudo`等命令来提升权限。
### 26、Linux中，如何通过命令定位Linux服务器下的日志？
在Linux服务器下定位日志文件，通常需要根据你所使用的服务或应用程序来确定日志文件的位置。以下是一些常见的日志文件位置以及如何使用命令来查找它们：
1. **标准日志位置**：
    - `/var/log`：大多数Linux发行版的系统日志文件都存放在这个目录下。
2. **使用`find`命令查找日志文件**：  
    如果你不确定日志文件的具体位置，可以使用`find`命令在整个系统中搜索。例如，要搜索所有扩展名为`.log`的文件，可以使用：
```bash
find / -type f -name "*.log"
```
    这条命令会从根目录`/`开始搜索，查找所有文件名为`.log`结尾的文件。
3. **使用`grep`命令搜索日志内容**：  
    如果你知道日志文件中可能包含某些关键词，可以使用`grep`命令来搜索这些关键词。例如，要搜索所有包含`error`关键词的日志文件，可以使用：
```bash
grep -r "error" /var/log/
```
    这条命令会在`/var/log/`目录下递归搜索所有包含`error`关键词的文件。
4. **查看特定服务的日志**：
    - 对于Apache Web服务器，日志文件通常位于`/var/log/apache2/`或`/var/log/httpd/`。
    - 对于Nginx Web服务器，日志文件通常位于`/var/log/nginx/`。
    - 对于MySQL数据库，日志文件通常位于`/var/log/mysql/`。
5. **使用`tail`或`less`命令查看日志内容**：  
    找到日志文件后，可以使用`tail`或`less`命令来查看文件内容。例如，要查看`/var/log/syslog`文件的最后几行，可以使用：
```shell
tail /var/log/syslog
```
    或者，使用`less`命令来分页查看文件内容：
```shell
less /var/log/syslog
```
通过以上方法，你可以有效地在Linux服务器下定位并查看日志文件。
### 27、Linux匹配 nginx.log 中 404 和 500 有多少行？
```shell
grep -E '404|500' nginx.log | wc -l
```
### 28、Linux，统计 nginx.log中 出现次数最多的 IP
```shell
cat nginx.log | awk '{print $1}' | sort | uniq -c | sort -rn | head -n 1
`cat` 命令用于将 nginx.log 文件的内容输出，然后使用 `awk` 命令获取每行的第一个字段，即 IP 地址。接着，使用 `sort` 命令将 IP 地址排序，再使用 `uniq` 命令计算每个 IP 地址出现的次数。此时，得到的结果中，IP 地址和出现次数的顺序是反过来的，因此需要使用 `sort` 命令进行倒序排序，并使用 `head` 命令获取第一行，即出现次数最多的 IP。
```
### 29、将 nginx.log 中的 topics 后面的数字替换成 number
```shell
sed -E 's/topics\/[0-9]*/topics\/number/g' nginx.log  
-E 采用正则   's/old/new/g' 固定替换公式  
[0-9]匹配数字，[0-9]*匹配 0个或者多个数字  
\/ 表示 / 符号，进行转义
```
### 30、将nginx.log 中的 ip 地址横向打印
```shell
awk '{print $1}' nginx.log  | sed -e ':1;N;s/\n/|/g;t1'  
:1 ~ t1 指设置一个标记，若成功，则重复执行  
N指追加前面的IP 例如：216.x.x.x \n 216.x.x.x  
s/\n/|/g 指将换行符替换成|
```
### 31、如何通过 Linux 命令查看后台执行的任务？需要给出详细的参数说明。
答案：
使用方法：要查看后台执行的任务，可以使用`jobs`命令。这个命令用于显示当前终端会话中正在运行或挂起的后台任务。
命令：`jobs [-lprs] or jobspec`
以下是`jobs`命令的详细参数说明：
- `-l`：显示任务的进程号及其状态。
- `-p`：显示任务的进程号。
- `-r`：仅显示运行中的任务。
- `-s`：仅显示挂起的任务。
- `jobspec`：可选参数，指定要查询的特定任务。它可以是任务的作业号（以`%`开头）或进程号。 

例如，运行以下命令可以显示当前终端会话中的所有后台任务及其进程号和状态：
- 命令： `jobs -l`
命令： `awk '{print $2}' 日志文件名`
- 在上面的命令中，你需要将`日志文件名`替换为你实际的日志文件名。
- 然后，`$2`表示提取每行的第二列。
- `print`命令将提取到的第二列内容打印出来
命令操作
- 如果只是显示运行中的后台任务，可以使用`-r`参数：
- 命令：`jobs -r`
- 类似地，如果只想显示挂起的后台任务，可以使用`-s`参数：
- 命令：`jobs -s`
### 32、如果你在面试过程中，被面试官问到：请说出你常用的十个 Linux 命令，并讲出它们的使用场景？你会如何回答呢？
- `ls`：用于列出当前目录下的文件和子目录。使用场景：浏览文件系统，查看目录中的内容。
- `cd`：用于进入到指定的目录。使用场景：切换工作目录，进入到其他目录以执行操作。
- `cat`：用于显示文件的内容。使用场景：查看文件的内容，合并文件。
- `less`：用于查看任意大小的文本文件，支持滚动翻页、搜索、标记等功能。
- `rm`：用于删除文件和目录。使用场景：删除不再需要的文件或目录。
- `cp`：用于复制文件和目录。使用场景：将文件复制到不同的位置，备份文件。
- `mv`：用于移动文件和目录，或者给文件和目录重命名。使用场景：移动文件到不同的位置，更改文件名。
- `chmod`：用于修改文件和目录的权限。使用场景：设置文件和目录的读、写、执行权限。
- `grep`：用于在文本中查找匹配的行。使用场景：通过过滤特定文本模式来查找特定信息。
- `ssh`：用于远程登录到其他计算机。使用场景：远程管理服务器，执行命令和操作。
### 33、如何查看日志里面有 error 关键字的日志记录，与这个记录前后三行的日志信息？请给出命令，并对命令进行说明。
答案：命令： `grep -C 3 "keyword" file_path`
- `grep`：文本搜索工具，可以接收管道传来的数据，这里是读取的文件的数据
- `-C n`：将匹配到关键字的前后n行一同输出
- `"keyword"`：预期被匹配到的关键字，在`egrep`或`-E`参数作用下，可以使用正则表达式
- `file_path`：将文件内容作为输入
### 34、请谈谈硬链接和软链接的区别
答案：
**定义不同**
- 软链接又叫符号链接，这个文件包含了另一个文件的路径名。可以是任意文件或目录，可以链接不同文件系统的文件。  
- 硬链接就是一个文件的一个或多个文件名。把文件名和计算机文件系统使用的节点号链接起来。因此我们可以用多个文件名与同一个文件进行链接，这些文件名可以在同一目录或不同目录。  

**限制不同**
- 硬链接只能对已存在的文件进行创建，不能对目录进行创建，也不能交叉文件系统进行硬链接的创建；  
- 软链接可对文件或目录创建，支持交叉文件系统，还可对不存在的文件或目录创建软链接；  

**影响不同**
- 删除一个硬链接文件并不影响其他有相同 inode 号的文件。  
- 而删除软链接并不影响被指向的文件，但若被指向的原文件被删除，则相关软连接被称为死链接。  

**命令不同**
- 创建软链接使用 ln -s 源文件 目标文件，注意其中源文件必须是绝对路径；  
- 创建软链接使用 ln 源文件 目标文件。
### 35、查看日志的前十行如何实现
面试题：在测试过程中，经常需要查看日志辅助定位问题，那么如果在服务器上查看日志的前十行如何实现？请写出对应命令，并给出对应的说明。
答案：
要在服务器上查看日志文件的前十行，你可以使用`head`命令。以下是`head`命令的使用方法和说明：
命令：`head [options] [file]`
其中，`options`是可选参数，可以根据需要选择使用。
以下是一些常用的参数： 
- `-n NUM`，`--lines=NUM`：显示文件的前`NUM`行，默认为显示前10行。
- `file`是要查看的文件名。

案例：
显示`logfile.txt`文件的前十行内容。  
- 命令：`head -n 10 logfile.txt`
显示系统的日志文件`/var/log/syslog`的前十行内容。  
- 命令：`head -n 10 /var/log/syslog`

总结  
`head`命令是一个非常有用的工具，可以用于快速查看日志、文本文件或任何其他类型的文件的开头部分。通过指定适当的参数，你可以按需显示任意数量的行数。


## Git篇
### 1、git经常用的几个命令，怎么切分支，怎么解决冲突？请详细说明
#### 常用命令
- 克隆项目：`git clone 仓库名称`
- 拉取代码：`git pull`
- 切出新分支：`git checkout -b 分支名称`
- 切换分支：`git checkout 分支名称`
- 提交代码：
```shell
git add 被添加代码  
git commit -m "提交记录信息"  
# 合并远程主分支  
git pull origin main  
git push
```
#### 解决冲突
1. 在push代码之前，先拉取主分支代码，并进行merge。
2. 如果产生冲，那么通过IDE 或者手动操作，解决冲突后再重新提交到自己分支。
3. 成功提交到个人分支后。再通过MR 合入主分支。